#include <horn.hpp>
#include <omp.h>

using namespace QOSM::Propagation;
using namespace QOSM::EM::Source;

Horn::Horn(double freq_GHz) : Source(freq_GHz){
    freq = freq_GHz * 1e9;
    k0 = 2*nc::constants::pi * freq / 299792458.;
    rotZ = 0.;
}

void Horn::init(std::vector<cplx> &inModes, const double &Lx, const double &Ly, const double &rotZ_deg, const size_t numPtsAperture){
    modes = inModes;
    const size_t numModes = modes.size();
    // apply rotation around src z-axis: example -> eg for the case of diagonal horn
    // minus because it is a rotation and not a change of frame
    rotZ = - rotZ_deg  * nc::constants::pi / 180.;
    std::array<double, 4> qRotZ{
        std::cos(rotZ * .5),
        0.,
        0.,
        std::sin(rotZ * .5)
    };
    Frame f_ap_src(Vec3(), qRotZ);
    aperture = std::make_shared<RectangularAperture>(freq, Lx, Ly, numModes, numPtsAperture, f_ap_src);
}

std::tuple<FieldNx3, FieldNx3, VectorNx3> Horn::apertureFields() const{
    FieldNx3 Ea(aperture->Xa.size());
    FieldNx3 Ha(aperture->Xa.size());
    std::cout << "Computing aperture fields ... ";
    for (int k=0; k<modes.size(); k++){
        Ea += aperture->Ek.at(k) * modes.at(k);
        Ha += aperture->Hk.at(k) * modes.at(k);
    }
    std::cout << "Done." << std::endl;
    
    VectorNx3 pts_aperture_local(aperture->Xa.size());  
    nc::Slice a(aperture->Xa.size());
    pts_aperture_local.put(a, 0, aperture->Xa);
    pts_aperture_local.put(a, 1, aperture->Ya);
    pts_aperture_local.put(a, 2, 0.);
    return std::make_tuple(aperture->frame.toRef(Ea), aperture->frame.toRef(Ha), aperture->frame.rot2Ref(pts_aperture_local));
}

FieldNx3 Horn::eField(const VectorNx3 &r_pts_src) const{
    auto eField = aperture->eField(modes, r_pts_src);
    return eField;
}

FieldNx3 Horn::hField(const VectorNx3 &r_pts_src) const{ 
    auto hField = aperture->hField(modes, r_pts_src);
    return hField;
}

std::tuple<FieldNx3, FieldNx3, VectorNx3> Horn::fields(const VectorNx3 &r_pts_src) const{
    return aperture->fields(modes, r_pts_src);
}